<?php
namespace Drupal\miniorange_2fa\form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\miniorange_2fa\MoAuthUtilities;
use Drupal\Component\Render\FormattableMarkup;

class UserMfaSetup extends FormBase {
    public function getFormId() {
        return "mo_mfa_form";
    }

    public function buildForm(array $form, FormStateInterface $form_state) {
        $variables_and_values = array(
          'mo_auth_enable_two_factor',
        );
        $mo_db_values = MoAuthUtilities::miniOrange_set_get_configurations( $variables_and_values, 'GET' );
        \Drupal::service('page_cache_kill_switch')->trigger();

        $custom_attribute = MoAuthUtilities::get_users_custom_attribute(\Drupal::currentUser()->id());

        if( isset( $custom_attribute[0]->activated_auth_methods ) ) {
            $form['mo_2fa_method'] = array(
                '#type' => 'fieldset',
            );

            $form['mo_2fa_method']['mo_configured_2fa_methods'] = array(
                '#markup' => t('<strong>Configured 2FA method: </strong>' . $custom_attribute[0]->activated_auth_methods . '<br><br>')
            );

            if( in_array( $custom_attribute[0]->activated_auth_methods, MoAuthUtilities::mo_TOTP_2fa_mentods() ) && isset( $custom_attribute[0]->qr_code_string ) && !empty( $custom_attribute[0]->qr_code_string ) ) {
                $qrCode = $custom_attribute[0]->qr_code_string;
                $image = new FormattableMarkup('<img src="data:image/jpg;base64, ' . $qrCode . '"/>', [':src' => $qrCode]);

                $form['mo_2fa_method']['mo_qr_code_details'] = array(
                    '#markup' => t('<ul><li>You can scan the below QR Code from ' . ucwords( strtolower( $custom_attribute[0]->activated_auth_methods ) ) . ' app to configure the 2FA on multiple devices.</li></ul>')
                );

                $form['mo_2fa_method']['mo_scan_qr_code_google_authenticator'] = array(
                    '#markup' => $image = isset($image) ? $image : '',
                );
            }
        }

        /**
         * We can't let you see the settings if you are not admin and trying to edit 2FA for some other user
         */
        if( !$mo_db_values['mo_auth_enable_two_factor'] || !MoAuthUtilities::isUserCanSee2FASettings() ) {
            return $form;
        }

        $form['mo_2fa_method']['mo_mfa_enable'] = array(
            '#type'=> 'checkbox',
            '#title'=> t('Enable MFA for your account'),
            '#default_value'=> isset( $custom_attribute[0]->enabled ) ? $custom_attribute[0]->enabled : FALSE,
            '#description'=> t('<strong>Note:</strong> You will not be prompted for 2FA if you disable this'),
            '#prefix' => '<hr>'
        );

        $form['mo_2fa_method']['mo_mfa_form_save'] = array(
            '#type'=>'submit',
            '#value'=> t('Submit'),
            '#button_type' => 'primary',
        );

        return $form;
    }

    public function submitForm(array &$form, FormStateInterface $form_state) {
        $formValues = $form_state->getValues();
        if( $formValues["mo_mfa_enable"] === 0 ) {
            if( !MoAuthUtilities::isSkipNotAllowed() ) {
                \Drupal::messenger()->addError(t("You are not allowed to disable 2FA. Please contact your site administrator"));
                return;
            }
        }
        MoAuthUtilities::updateMfaSettingsForUser(\Drupal::currentUser()->id(), $formValues["mo_mfa_enable"] == 1);
        \Drupal::messenger()->addStatus(t('2FA settings are updated for your account'));
    }
}